#include <avr/io.h>
#include <util/delay.h>
#include "my_utils.h"


/* ------------------------------------------------ FUNCTII AUX ----------------------------------------------- */
void set_environment() {
	PORTA = 0;
	PORTB = 0;
	PORTC = 0;
	
	/* Setarea butoanelor de control ca pini de intrare */
	DDRA &= ~((1 << NORMAL_MODE_BUTTON) | 
			  (1 << RECORDING_MODE_BUTTON) |
			  (1 << PLAY_MODE_BUTTON) |
			  (1 << OCTAVE_PLUS) |
			  (1 << OCTAVE_MINUS));	
	PORTA |= ((1 << NORMAL_MODE_BUTTON) | 
			  (1 << RECORDING_MODE_BUTTON) |
			  (1 << PLAY_MODE_BUTTON) |
			  (1 << OCTAVE_PLUS) |
			  (1 << OCTAVE_MINUS));

	/* Setarea LED-urilor de control ca pini de iesire */ 
	DDRA |= (normal_mode_output << PA0);
	PORTA &= ~(normal_mode_output  << PA0);
	DDRB |= ((record_mode_output << PB0) | (play_mode_output << PB0));
	PORTB &= ~((record_mode_output << PB0) | (play_mode_output << PB0));
	
	/* Setarea butoanelor de pe porturile B, C si D (clapele pianului) ca pini de intrare */
	DDRB &= ~((1 << DO) | (1 << LA_DIEZ));
	PORTB |= ((1 << DO) | (1 << LA_DIEZ));
	DDRC &= ~((1 << DO_DIEZ) | (1 << RE) | (1 << RE_DIEZ) | (1 << MI) | (1 << FA) | (1 << FA_DIEZ) | (1 << SOL));
	PORTC |= ((1 << DO_DIEZ) | (1 << RE) | (1 << RE_DIEZ) | (1 << MI) | (1 << FA) | (1 << FA_DIEZ) | (1 << SOL));
	DDRD &= ~((1 << SOL_DIEZ) | (1 << LA) | (1 << SI));
	PORTD |= ((1 << SOL_DIEZ) | (1 << LA) | (1 << SI));
	
	/* Setarea pinului pentru buzzer ca iesire si dezactivarea lui */
	DDRD |= (1 << BUZZER);
	PORTD |= (1 << BUZZER); 
	
	/* Initializarea LED-urilor (DEFAULT: normal_mode cu octava 1) */
	PORTA |= (1 << NORMAL_MODE_BLUE);
	PORTB |= (1 << RECORDING_MODE_RED);
	PORTB |= (1 << PLAY_MODE_RED);
}

int my_max_function(int a, int b) {
	if (a > b)
		return a;
	return b;
}

int my_min_function(int a, int b) {
	if (a < b)
		return a;
	return b;
}

/* ------------------------------------------------ FUNCTII LED-URI ------------------------------------------------ */

void change_octave_colors() {
	/* Stinge LED-ul */
	PORTA &= ~(normal_mode_output << PA0);
	
	/* Aprinde LED-ul in culoarea corespunzatoare octavei curente */
	switch(current_octave) {
		case 1:
			/* Octava 1 => Albastru */
			PORTA |= (1 << NORMAL_MODE_BLUE);
			break;
		case 2:
			/* Octava 2 => Verde */
			PORTA |= (1 << NORMAL_MODE_GREEN);
			break;
		case 3:
			/* Octava 3 => Cyan */
			PORTA |= (1 << NORMAL_MODE_BLUE);
			PORTA |= (1 << NORMAL_MODE_GREEN);
			break;
		case 4:
			/* Octava 4 => Magenta */
			PORTA |= (1 << NORMAL_MODE_BLUE);
			PORTA |= (1 << NORMAL_MODE_RED);
			break;
		case 5:
			/* Octava 5 => Galben */
			PORTA |= (1 << NORMAL_MODE_RED);
			PORTA |= (1 << NORMAL_MODE_GREEN);
			break;
		case 6:
			/* Octava 6 => Alb */
			PORTA |= (1 << NORMAL_MODE_RED);
			PORTA |= (1 << NORMAL_MODE_GREEN);
			PORTA |= (1 << NORMAL_MODE_BLUE);
		default:
			return;
	}
}

void change_colors() {
	/* Actualizare led mod normal + octava curenta */
	PORTA &= ~(normal_mode_output << PA0);
	if (normal_mode == ACTIVE)
		change_octave_colors();
	else
		PORTA |= (1 << NORMAL_MODE_RED);
	
	/* Actualizare led mod inregistrare melodie */
	PORTB &= ~(record_mode_output << PB0);
	if (recording_mode == DISABLED)
		PORTB |= (1 << RECORDING_MODE_RED);
	else if (recording_mode == READY) {
		PORTB |= (1 << RECORDING_MODE_BLUE);
		change_octave_colors();
	}
	else if (recording_mode == ACTIVE) {
		PORTB |= (1 << RECORDING_MODE_GREEN);
		change_octave_colors();
	}
	
	/* Actualizare led mod redare melodie */
	PORTB &= ~(play_mode_output << PB0);
	if (play_mode == DISABLED)
		PORTB |= (1 << PLAY_MODE_RED);
	else if (play_mode == READY)
		PORTB |= (1 << PLAY_MODE_BLUE);
	else if (play_mode == ACTIVE)
		PORTB |= (1 << PLAY_MODE_GREEN);
}


/* ------------------------------------------------ FUNCTII CLAPE ------------------------------------------------ */
void play_sound(long frequency, long length) 
{
	long numCycles, i;
	double delayValue;
 
	delayValue = 1000000 / frequency / 2;
	numCycles = frequency * length / 1000;
	for (i = 0; i < numCycles; i++) {
		PORTD |= (1 << BUZZER);
		_delay_us(delayValue);
		PORTD &= ~(1 << BUZZER);
		_delay_us(delayValue);
	}
}

double check_pressed_note() {
	if ((PINB & (1 << DO)) == 0) {
		play_sound(DO_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return DO_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << DO_DIEZ)) == 0) {
		play_sound(DO_DIEZ_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return DO_DIEZ_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << RE)) == 0) {
		play_sound(RE_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return RE_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << RE_DIEZ)) == 0) {
		play_sound(RE_DIEZ_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return RE_DIEZ_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << MI)) == 0) {
		play_sound(MI_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return MI_FREQUENCY[current_octave];
	}	
	
	if ((PINC & (1 << FA)) == 0) {
		play_sound(FA_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return FA_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << FA_DIEZ)) == 0) {
		play_sound(FA_DIEZ_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return FA_DIEZ_FREQUENCY[current_octave];
	}
	
	if ((PINC & (1 << SOL)) == 0) {
		play_sound(SOL_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return SOL_FREQUENCY[current_octave];
	}
	
	if ((PIND & (1 << SOL_DIEZ)) == 0) {
		play_sound(SOL_DIEZ_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return SOL_DIEZ_FREQUENCY[current_octave];
	}
	
	if ((PIND & (1 << LA)) == 0) {
		play_sound(LA_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return LA_FREQUENCY[current_octave];
	}
	
	if ((PINB & (1 << LA_DIEZ)) == 0) {
		play_sound(DO_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return DO_FREQUENCY[current_octave];
	}
	
	if ((PIND & (1 << SI)) == 0) {
		play_sound(SI_FREQUENCY[current_octave], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		return SI_FREQUENCY[current_octave];
	}

	return 0;
}


/* ------------------------------------------------ FUNCTII PIAN ------------------------------------------------ */
int check_changed_mode() {
	if ((PINA & (1 << NORMAL_MODE_BUTTON)) == 0) {
		if (normal_mode == 0)
			normal_mode = ACTIVE;
		else
			normal_mode = DISABLED;
		recording_mode = DISABLED;
		play_mode = DISABLED;
		change_colors();
		return 1;
	}
	
	if ((PINA & (1 << RECORDING_MODE_BUTTON)) == 0) {
		normal_mode = DISABLED;
		recording_mode = (recording_mode + 1) % 3;
		play_mode = DISABLED;
		if (recording_mode == DISABLED && last_note_index > 0)
			recorded_melody[last_note_index++] = -1;
		else if (recording_mode == READY) {
			last_note_index = 0;
		}
		change_colors();
		return 1;
	}
	
	if ((PINA & (1 << PLAY_MODE_BUTTON)) == 0) {
		normal_mode = DISABLED;
		recording_mode = DISABLED;
		play_mode = (play_mode + 1) % 3;
		change_colors();
		return 1;
	}
	
	return 0;
}

int check_changed_octave() {
	/* Se poate schimba octava curenta doar daca pianul se afla in modul normal
	de functionare sau in modul de inregistrare */
	if (!(normal_mode != DISABLED || recording_mode != DISABLED))
		return 0;
	
	if ((PINA & (1 << OCTAVE_PLUS)) == 0) {
		current_octave = my_min_function(current_octave + 1,  NOF_OCTAVES);
		change_colors();
		return 1;
	}
	
	if ((PINA & (1 << OCTAVE_MINUS)) == 0) {
		current_octave = my_max_function(current_octave - 1, 1);
		change_colors();
		return 1;
	}
	
	return 0;
}

void record_music() {
	/* Se retine frecventa notei care a fost apasata si se adauga in vector */
	double note_frequency = check_pressed_note();
	recorded_melody[last_note_index++] = note_frequency;
}

void play_recorded() {
	int i = 0;
	
	/* Reda pe rand fiecare nota din vector */
	while (i < last_note_index && recorded_melody[i] != -1) {
		play_sound(recorded_melody[i], SOUND_LENGTH);
		PORTD |= (1 << PD5);
		_delay_ms(250);
		
		/* Verific daca s-a oprit redarea melodiei intre timp */
		if ((PINA & (1 << PLAY_MODE_BUTTON)) == 0) {
			play_mode = DISABLED;
			change_colors();
			return;
		}
		i++;
	}
	
	/* Actualizez starea modului PLAY (ca sa evit redarea la infinit a melodiei) */
	play_mode = READY;
	change_colors();
}

void check_buttons() {
	int return_code;

	return_code = check_changed_mode();
	if (return_code != 0)
		return;
	
	return_code = check_changed_octave();
	if (return_code != 0)
		return;
	
	if (normal_mode == ACTIVE)
		check_pressed_note();
	else if (recording_mode == ACTIVE)
		record_music();
	else if (play_mode == ACTIVE)
		play_recorded();
	
	PORTD |= (1 << PD5);
}

/* ------------------------------------------------ MAIN ------------------------------------------------ */
int main(void) {
	
	set_environment();
	
	while (1) {
		check_buttons();
		_delay_ms(250);
	}
	
	return 0;
}
