/* Cauc Bianca-Mihaela 331CB - Pian electronic */

/* Stari ale unui mod de functionare */
#define DISABLED 0
#define READY    1
#define ACTIVE   2

/* Pini pentru LED-ul corespunzator modului normal de functionare + octava curenta */
#define NORMAL_MODE_RED        PA2
#define	NORMAL_MODE_GREEN      PA1
#define NORMAL_MODE_BLUE       PA0

/* Pini pentru LED-ul corespunzator modului de inregistrare */
#define RECORDING_MODE_RED     PB5
#define RECORDING_MODE_GREEN   PB6
#define RECORDING_MODE_BLUE    PB7

/* Pini pentru LED-ul corespunzator modului de redare */
#define PLAY_MODE_RED          PB2
#define PLAY_MODE_GREEN        PB3
#define PLAY_MODE_BLUE         PB4

/* Pini pentru butoanele de control */
#define NORMAL_MODE_BUTTON     PA7
#define RECORDING_MODE_BUTTON  PA6
#define PLAY_MODE_BUTTON       PA5
#define OCTAVE_PLUS            PA4
#define OCTAVE_MINUS           PA3

/* Pin pentru modulul de buzzer activ */
#define BUZZER                 PD5

/* Pini pentru clapele pianului */
#define DO        PB1
#define DO_DIEZ   PC6
#define RE        PC5
#define RE_DIEZ   PC4
#define MI        PC3
#define FA        PC2
#define FA_DIEZ   PC1
#define SOL       PC0
#define SOL_DIEZ  PD4
#define LA        PD1
#define LA_DIEZ   PB0
#define SI        PD0

#define NOF_OCTAVES  6
#define SOUND_LENGTH 300

/* Functii */
void set_environment();
int my_max_function(int a, int b);
int my_min_function(int a, int b);
int check_changed_mode();
int check_changed_octave();
void check_buttons();
void play_sound(long frequency, long length);
double check_pressed_note();
void change_octave_colors();
void change_colors();

/* Variabile pentru retinerea starilor modurilor de functionare si a octavei curente */
static unsigned int normal_mode = ACTIVE;
static unsigned int recording_mode = DISABLED;
static unsigned int play_mode = DISABLED;
static unsigned int current_octave = 1;

/* Variabila pentru retinerea melodiei inregistrate */
static double recorded_melody[50];
static int last_note_index = 0;

/* Frecventele notelor muzicale pentru primele 6 octave */
double DO_FREQUENCY[NOF_OCTAVES]       = { 16.352, 32.703, 65.406, 130.813, 261.626, 523.251 };
double DO_DIEZ_FREQUENCY[NOF_OCTAVES]  = { 17.324, 34.648, 69.296, 138.591, 277.183, 554.365 };
double RE_FREQUENCY[NOF_OCTAVES]       = { 18.354, 36.708, 73.416, 146.832, 293.665, 587.333 };
double RE_DIEZ_FREQUENCY[NOF_OCTAVES]  = { 19.445, 38.891, 77.782, 155.563, 311.127, 622.254 };
double MI_FREQUENCY[NOF_OCTAVES]       = { 20.602, 41.203, 82.407, 164.814, 329.628, 659.255 };
double FA_FREQUENCY[NOF_OCTAVES]       = { 21.827, 43.654, 87.307, 174.614, 349.228, 698.456 };
double FA_DIEZ_FREQUENCY[NOF_OCTAVES]  = { 23.125, 46.249, 92.499, 184.997, 369.994, 739.989 };
double SOL_FREQUENCY[NOF_OCTAVES]      = { 24.500, 48.999, 97.999, 195.998, 391.995, 783.991 };
double SOL_DIEZ_FREQUENCY[NOF_OCTAVES] = { 25.957, 51.913, 103.826, 207.652, 415.305, 830.609 };
double LA_FREQUENCY[NOF_OCTAVES]       = { 27.500, 55.000, 110.000, 220.000, 440.000, 880.000 };
double LA_DIEZ_FREQUENCY[NOF_OCTAVES]  = { 29.135, 58.270, 116.541, 233.082, 466.164, 932.328 };
double SI_FREQUENCY[NOF_OCTAVES]       = { 30.868, 61.735, 123.471, 246.942, 493.883, 987.767 };

/* Variabile pentru retinerea pinilor fiecarui LED */
int normal_mode_output = (1 << NORMAL_MODE_RED) + 
						 (1 << NORMAL_MODE_GREEN) + 
						 (1 << NORMAL_MODE_BLUE);
int record_mode_output = (1 << RECORDING_MODE_RED) + 
						 (1 << RECORDING_MODE_GREEN) + 
						 (1 << RECORDING_MODE_BLUE);
int play_mode_output = (1 << PLAY_MODE_RED) + 
					   (1 << PLAY_MODE_GREEN) + 
					   (1 << PLAY_MODE_BLUE);